﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
//using System.Windows.Forms.Design; // Not required for .NET Core 3 but may be required for .NET Framework apps hence retained.


namespace Common.MVVM.Messages
{
    //Any class can publish an event here 
    //Any class can subscribe to event so for particular type here 
    public sealed class EventAggregator : IEventAggregator
    {
        private Dictionary<Type, List<ActionRef>> _subscribers = new Dictionary<Type, List<ActionRef>>();

        //gatekeeper for subscriptions
        private readonly object _subscriptionLock = new object();

        public void Subscribe<T>(Action<T> handler)
        {
            lock (_subscriptionLock)
            {
                if (_subscribers.ContainsKey(typeof(T)))
                {
                    //already have type just add handler
                    var subscription = _subscribers[typeof(T)];
                    subscription.Add(new ActionRef(handler));
                }
                else
                {
                    var handlers = new List<ActionRef>();
                    handlers.Add(new ActionRef(handler));
                    _subscribers[typeof(T)] = handlers;
                }
            }
        }


        public void Unsubscribe<T>(Action<T> handler)
        {
            lock (_subscriptionLock)
            {
                if (_subscribers.ContainsKey(typeof(T)))
                {
                    var handlers = _subscribers[typeof(T)];
                    foreach (var entry in handlers
                        .Where(en => en.Target.Target == handler.Target && en.Target.Method == handler.Method))
                    {
                        //there should only ever be one but if not this will clean up all entries otherwise the subscribe would fail to unsubscribe
                        handlers.Remove(entry);
                    }

                    if (handlers.Count == 0) //we have no more subscribers to this
                    {
                        _subscribers.Remove(typeof(T));
                    }
                }
            }
        }

        public void Publish<T>(T evt)
        {
            lock (_subscriptionLock)
            {
                if (_subscribers.ContainsKey(typeof(T)))
                {
                    var handlers = _subscribers[typeof(T)];
                    foreach (var handler in handlers)
                    {
                        ((Action<T>)handler.Target).Invoke(evt);
                    }
                }
            }
        }


        /// <summary>
        /// Holds the subscribers via a weak reference.
        /// </summary>

        private Dictionary<Type, List<WeakReference>> _subscriberCache = new Dictionary<Type, List<WeakReference>>();

        private readonly SemaphoreSlim _lock = new SemaphoreSlim(1);


        public void PublishEvent<TEventType>(TEventType eventToPublish)
        {
            var subscriberType = typeof(IEventSubscriber<>).MakeGenericType(typeof(TEventType));

            List<WeakReference> subscribers = GetSubscriberList(subscriberType);

            List<WeakReference> subscribersToBeRemoved = new List<WeakReference>();

            foreach (var weakSubscriber in subscribers)
            {
                if (weakSubscriber == null) continue;

                if (weakSubscriber.IsAlive)
                {
                    IEventSubscriber<TEventType> subscriber = (IEventSubscriber<TEventType>)weakSubscriber.Target;

                    InvokeSubscriberEvent<TEventType>(eventToPublish, subscriber);
                }
                else
                {
                    subscribersToBeRemoved.Add(weakSubscriber);
                }
            }

            if (subscribersToBeRemoved.Any())
            {
                lock (_lock)
                {
                    foreach (var remove in subscribersToBeRemoved)
                    {
                        subscribers.Remove(remove);
                    }
                }
            }


        }

        public void SubscribeEvent(object subscriber)
        {
            lock (_lock)
            {
                var subscriberTypes = subscriber.GetType().GetInterfaces()
                    .Where(i => i.IsGenericType && i.GetGenericTypeDefinition() == typeof(IEventSubscriber<>));

                WeakReference weakReference = new WeakReference(subscriber);

                foreach (var subscriberType in subscriberTypes)
                {
                    List<WeakReference> subscribers = GetSubscriberList(subscriberType);

                    subscribers.Add(weakReference);
                }
            }
        }

 

        private List<WeakReference> GetSubscriberList(Type subscriberType)
        {
            bool found = _subscriberCache.TryGetValue(subscriberType, out List<WeakReference> subscribersList);

            if (!found)
            {
                subscribersList = new List<WeakReference>();

                _subscriberCache.Add(subscriberType, subscribersList);
            }

            return subscribersList;
        }


        private void InvokeSubscriberEvent<TEventType>(TEventType eventToPublish, IEventSubscriber<TEventType> subscriber)
        {
            //Synchronize the invocation of method 

            SynchronizationContext syncContext = SynchronizationContext.Current;

            if (syncContext == null)
            {
                syncContext = new SynchronizationContext();

            }

            syncContext.Post(s => subscriber.OnEventHandler(eventToPublish), null);
        }




    }


}
