﻿using Common.MVVM.Messages;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using Color = System.Drawing.Color;
using System.Reflection;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using Common.Unity;

namespace Common.MVVM.ViewModels
{
    /// <summary>
    /// 
    /// </summary>
    /// <typeparam name="T">Model</typeparam>
    public abstract class ViewModelBase<T> : DependencyObject, INotifyPropertyChanged, IViewModelBase
    {

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }


        protected bool SetField<TV>(ref TV field, TV value, string propertyName)
        {
            if (EqualityComparer<TV>.Default.Equals(field, value)) return false;
            field = value;
            OnPropertyChanged(propertyName);
            return true;
        }

        // End of From BindableBase originally



        /// <summary>
        /// Runs a task asynchronously when the calling method return type cannot be Task (e.g. in a constructor).
        /// </summary>
        /// <param name="task">A task to run asynchronously.</param>
        protected void RunAsyncTask(Task task) => System.Windows.Application.Current.Dispatcher.Invoke(async () => await task);




        /// <summary>
        /// Holds the Model for the ViewModel.
        /// </summary>
        public T Model
        {
            get { return (T)GetValue(ModelProperty); }
            set { SetValue(ModelProperty, value); }
        }

        
        /// <summary>
        /// Gets or sets the Model object.
        /// </summary>
        public static readonly DependencyProperty ModelProperty =
                DependencyProperty.Register("Model", typeof(T), typeof(ViewModelBase<T>),
                new PropertyMetadata((o, e) =>
                {
                    var screenWithModel = (ViewModelBase<T>)o;
                    screenWithModel.OnModelChanged((T)e.OldValue, (T)e.NewValue);
                }));










        /// <summary>
        /// Invoked when the Model changes.
        /// /// </summary>
        /// <param name="oldValue">Previous Model reference.</param>
        /// <param name="newValue">New Model reference.</param>
        protected virtual void OnModelChanged(T oldValue, T newValue)
        {
 
        }



  


        /// <summary>
        /// Gets the shell view model from this view model's hierarchy.
        /// </summary>
        /// <remarks>The system needs to be setup so that a view model has a parent and so on</remarks>
        /// <remarks>up a chain so that a hierarchy is built with ShellViewModel at the summit.</remarks>
        /// <remarks>The hierarchy is a logical one in that the ShellViewModel contains child view models</remarks>
        /// <remarks>and these child view models themselves have child view models.</remarks>
        /// <remarks>Similarly, the ShellView contains child views usually UserControls</remarks>
        /// <remarks>and these child views themselves contain one or more child views (UserControls).</remarks>
        /// <returns>ShellViewModel or null if cannot be found.</returns>
        protected IViewModelBase GetShellViewModel()
        {
            string msg = "thisViewModel = " + this + Environment.NewLine + Environment.NewLine;

            if (ParentViewModel == null)
            {
                msg += "ParentViewModel = null";
                return null;
            }

            IViewModelBase viewModelBase = ParentViewModel;
            msg += "ParentViewModel = " + viewModelBase + Environment.NewLine + Environment.NewLine;

            do
            {
                if (viewModelBase.ParentViewModel == null)
                {
                    // If parent is null the viewModelBase should be ShellViewModel as it is at the top of the hierarchy.
                    string viewModelBaseText = viewModelBase.ToString();

                    if (!viewModelBaseText.Contains("ShellViewModel"))
                    {
                        return null; 
                    }

                    return viewModelBase; // This is ShellViewModel.
                }

                viewModelBase = viewModelBase.ParentViewModel;
                msg += "ParentViewModel = " + viewModelBase + Environment.NewLine + Environment.NewLine;

            } while (true);
        }






        /// <summary>
        /// Holds a value in milliseconds for an additional time delay until removal of a footer message begins.
        /// </summary>
        private double _addTimeDelayUntilRemoval;


        /// <summary>
        /// Holds the parent view model.
        /// </summary>
        public IViewModelBase ParentViewModel { get; set; }


        /// <summary>
        /// Holds the child ribbon view model.
        /// </summary>
        public IViewModelBase ChildRibbonViewModel { get; set; }

        public virtual void ResizeChildView(object obj)
        {
            
        }


        protected ViewModelBase()
        {
            Model = UnityService.Resolve<T>();
        }




        public virtual void Initialize()
        {
            if (EventProvider.EventsHub != null)
            {
                EventProvider.EventsHub.SubscribeEvent(this);
            }
        }


        protected TU CreateView<TU>(IViewModelBase viewModel)
        {
            TU view = default(TU);

            Type type = viewModel.GetType();
            string viewModelName = type.ToString();

            if (viewModelName.Contains("Model"))
            {
                string viewName = viewModelName.Replace("Model", string.Empty, false, System.Globalization.CultureInfo.InvariantCulture);
                string assemblyName = string.Empty;

                Assembly assem = Assembly.GetAssembly(viewModel.GetType());
                System.Runtime.Remoting.ObjectHandle viewHandle = Activator.CreateInstance(assem.FullName, viewName);
                object testView = viewHandle.Unwrap();

                Type testViewType = testView.GetType().BaseType;

                PropertyInfo propertyInfo = testViewType.GetProperty("DataContext");

                // Sets the view's DataContext property to the viewmodel.
                propertyInfo.SetValue(testView, this);

                view = (TU)testView;
            }

            return view;
        }



















        private bool IsInstanceOfGenericType(Type genericType, object instance)
        {
            Type type = instance.GetType();
            while (type != null)
            {
                if (type.IsGenericType && type.GetGenericTypeDefinition() == genericType)
                {
                    return true;
                }
                type = type.BaseType;
            }
            return false;
        }






    }


  

}
