﻿using System;
using System.Windows;
using System.Windows.Controls;

namespace Common.MVVM.ViewModels
{
    /// <summary>
    /// Base class of view model that has a view that is a User Control.
    /// </summary>
    /// <typeparam name="T">Model.</typeparam>
    public class ViewModelWithUserControlBase<T> : ViewModelBase<T>, IViewModelWithUserControlBase where T: class
    {
        private bool _isBusy;
        private UserControl _userControl;

        /// <summary>
        /// Holds the value of whether the code is busy and thus show BusyIndicator.
        /// </summary>
        public bool IsBusy
        {
            get { return _isBusy; }
            set { SetField(ref _isBusy, value, nameof(IsBusy)); }
        }

  
        public UserControl UserControlView
        {
            get { return _userControl; }
            private set { _userControl = value; }
        }


        public UserControl GetUserControlView()
        {
            if (_userControl == null)
            {
                _userControl = CreateView<UserControl>(this);
            }

            return _userControl;
        }

        public override void Initialize()
        {
            base.Initialize();

        }
        

        public void Close()
        {
            _userControl.DataContext = null;
            _userControl = null;
        }

        /// <summary>
        /// Requests the user to override a denied action by showing a message box for authorization.
        /// </summary>
        /// <param name="messageText"></param>
        /// <param name="messageBoxCaption">Can be null.</param>
        /// <returns>True if override is authorized, otherwise false - do not override.</returns>
        protected bool RequestUserOverrideAuthorization(string messageText, string messageBoxCaption)
        {
            if (string.IsNullOrWhiteSpace(messageText)) throw new ArgumentNullException(nameof(messageText));

            MessageBoxResult messageBoxResult = MessageBox.Show(messageText, messageBoxCaption, MessageBoxButton.YesNoCancel, MessageBoxImage.Warning);

            if (messageBoxResult == MessageBoxResult.Yes)
            {
                return true;
            }

            if (messageBoxResult == MessageBoxResult.No || messageBoxResult == MessageBoxResult.Cancel)
            {
                return false;
            }

            throw new InvalidOperationException("Unexpected message box result");
        }


        /// <summary>
        /// Requests the user to answer a question by showing a message box for an answer.
        /// </summary>
        /// <param name="messageText"></param>
        /// <param name="messageBoxCaption">Can be null.</param>
        /// <returns>True if answer is yes, otherwise false meaning no.</returns>
        protected bool RequestUserYesNoQuestion(string messageText, string messageBoxCaption)
        {
            if (string.IsNullOrWhiteSpace(messageText)) throw new ArgumentNullException(nameof(messageText));

            MessageBoxResult messageBoxResult = MessageBox.Show(messageText, messageBoxCaption, MessageBoxButton.YesNo, MessageBoxImage.Warning);

            if (messageBoxResult == MessageBoxResult.Yes)
            {
                return true;
            }

            if (messageBoxResult == MessageBoxResult.No)
            {
                return false;
            }

            throw new InvalidOperationException("Unexpected message box result");
        }






    }
}
